@@ -12,18 +12,17 @@
 //! let datetime = iso8601::datetime("2015-06-26T16:43:23+0200").unwrap();
 //! ```
 
+#![allow(clippy::uninlined_format_args)]
 #![deny(
     missing_copy_implementations,
     trivial_casts,
     trivial_numeric_casts,
     unsafe_code,
     unused_import_braces,
-    unused_qualifications
-)]
-#![warn(
-    // missing_docs,
-    clippy::doc_markdown
+    unused_qualifications,
+    missing_docs
 )]
+#![warn(clippy::doc_markdown)]
 #![no_std]
 
 #[cfg(any(feature = "std", test))]
@@ -38,12 +37,19 @@ use core::default::Default;
 use core::str::FromStr;
 
 mod display;
-mod parsers;
+pub mod parsers;
+
+#[cfg(feature = "chrono")]
+mod chrono;
+
+#[cfg(feature = "serde")]
+mod serde;
 
 #[cfg(test)]
 mod assert;
 
 /// A date, can hold three different formats.
+#[allow(missing_docs)]
 #[derive(Eq, PartialEq, Debug, Copy, Clone)]
 pub enum Date {
     /// consists of year, month and day of month
@@ -54,7 +60,7 @@ pub enum Date {
     Ordinal { year: i32, ddd: u32 },
 }
 
-/// A time object
+/// A time object.
 #[derive(Eq, PartialEq, Debug, Copy, Clone, Default)]
 pub struct Time {
     /// a 24th of a day
@@ -65,29 +71,52 @@ pub struct Time {
     pub second: u32,
     /// everything after a `.`
     pub millisecond: u32,
-    /// depends on where you're at
+    /// the hour part of the timezone offset from UTC
     pub tz_offset_hours: i32,
+    /// the minute part of the timezone offset from UTC
     pub tz_offset_minutes: i32,
 }
 
-/// Compound struct, holds Date and Time
+/// Compound struct, holds Date and Time.
 #[derive(Eq, PartialEq, Debug, Copy, Clone, Default)]
 pub struct DateTime {
+    /// The date part
     pub date: Date,
+    /// The time part
     pub time: Time,
 }
 
 /// A time duration.
+
+/// Durations:
+/// <https://www.rfc-editor.org/rfc/rfc3339#page-13>
+///    dur-second        = 1*DIGIT "S"
+///    dur-minute        = 1*DIGIT "M" [dur-second]
+///    dur-hour          = 1*DIGIT "H" [dur-minute]
+///    dur-time          = "T" (dur-hour / dur-minute / dur-second)
+///    dur-day           = 1*DIGIT "D"
+///    dur-week          = 1*DIGIT "W"
+///    dur-month         = 1*DIGIT "M" [dur-day]
+///    dur-year          = 1*DIGIT "Y" [dur-month]
+///    dur-date          = (dur-day / dur-month / dur-year) [dur-time]
+///    duration          = "P" (dur-date / dur-time / dur-week)
 #[derive(Eq, PartialEq, Debug, Copy, Clone)]
 pub enum Duration {
-    /// consists of year, month, day, hour, minute and second units
+    /// A duration specified by year, month, day, hour, minute and second units
     YMDHMS {
+        /// Number of calendar years
         year: u32,
+        /// Number of months
         month: u32,
+        /// Number of days
         day: u32,
+        /// Number of hours
         hour: u32,
+        /// Number of minutes
         minute: u32,
+        /// Number of seconds
         second: u32,
+        /// Number of milliseconds
         millisecond: u32,
     },
     /// consists of week units
@@ -95,6 +124,7 @@ pub enum Duration {
 }
 
 impl Duration {
+    /// Whether this duration represents a zero duration.
     pub fn is_zero(&self) -> bool {
         *self
             == Duration::YMDHMS {
@@ -111,6 +141,11 @@ impl Duration {
 }
 
 impl Time {
+    /// Change this time's timezone offset.
+    ///
+    /// # Arguments
+    ///
+    /// * `tzo` - A tuple of `(hours, minutes)` specifying the timezone offset from UTC.
     pub fn set_tz(&self, tzo: (i32, i32)) -> Time {
         let mut t = *self;
         t.tz_offset_hours = tzo.0;
@@ -221,7 +256,7 @@ pub fn date(string: &str) -> Result<Date, String> {
     if let Ok((_, parsed)) = parsers::parse_date(string.as_bytes()) {
         Ok(parsed)
     } else {
-        Err(format!("Parser Error: {}", string))
+        Err(format!("Failed to parse date: {}", string))
     }
 }
 
@@ -243,7 +278,7 @@ pub fn time(string: &str) -> Result<Time, String> {
     if let Ok((_, parsed)) = parsers::parse_time(string.as_bytes()) {
         Ok(parsed)
     } else {
-        Err(format!("Parser Error: {}", string))
+        Err(format!("Failed to parse time: {}", string))
     }
 }
 
@@ -262,7 +297,7 @@ pub fn datetime(string: &str) -> Result<DateTime, String> {
     if let Ok((_left_overs, parsed)) = parsers::parse_datetime(string.as_bytes()) {
         Ok(parsed)
     } else {
-        Err(format!("Parser Error: {}", string))
+        Err(format!("Failed to parse datetime: {}", string))
     }
 }
 
@@ -301,6 +336,6 @@ pub fn duration(string: &str) -> Result<Duration, String> {
     if let Ok((_left_overs, parsed)) = parsers::parse_duration(string.as_bytes()) {
         Ok(parsed)
     } else {
-        Err(format!("Parser Error: {}", string))
+        Err(format!("Failed to parse duration: {}", string))
     }
 }
