@@ -1,12 +1,12 @@
-//! This module is strictly internal.
+//! The low-level parsers for date, datetime, duration and time.
 //!
-//! These functions are used by `date()`, `time()` and `datetime()`.
-//! They are currently not private, because the need to be accessible,
-//! but are not useful by themselves.
+//! The top-level functions [`date`][`crate::date`], [`datetime`][`crate::datetime`],
+//! [`duration`][`crate::duration`] and [`time`][`crate::time`]
+//! provide convenient wrappers around the low-level parsers,
+//! but throw away leftover input on success.
 //!
-//! Please refer to the top-level functions instead, as they offer a better abstraction.
-//!
-//! **These functions may be made private later.**
+//! Using the low-level functions provided here allows to recover leftover input
+//! or to combine these parsers with other parser combinators.
 
 use core::str;
 
@@ -54,17 +54,6 @@ fn take_n_digits(i: &[u8], n: usize) -> IResult<&[u8], u32> {
     Ok((i, res))
 }
 
-fn take_m_to_n_digits(i: &[u8], m: usize, n: usize) -> IResult<&[u8], u32> {
-    let (i, digits) = take_while_m_n(m, n, is_digit)(i)?;
-
-    let s = str::from_utf8(digits).expect("Invalid data, expected UTF-8 string");
-    let res = s
-        .parse()
-        .expect("Invalid string, expected ASCII representation of a number");
-
-    Ok((i, res))
-}
-
 fn n_digit_in_range(
     i: &[u8],
     n: usize,
@@ -79,21 +68,6 @@ fn n_digit_in_range(
     }
 }
 
-fn m_to_n_digit_in_range(
-    i: &[u8],
-    m: usize,
-    n: usize,
-    range: impl core::ops::RangeBounds<u32>,
-) -> IResult<&[u8], u32> {
-    let (new_i, number) = take_m_to_n_digits(i, m, n)?;
-
-    if range.contains(&number) {
-        Ok((new_i, number))
-    } else {
-        Err(Err::Error(Error::new(i, nom::error::ErrorKind::Eof)))
-    }
-}
-
 fn sign(i: &[u8]) -> IResult<&[u8], i32> {
     map(alt((tag(b"-"), tag(b"+"))), |s: &[u8]| match s {
         b"-" => -1,
@@ -101,14 +75,6 @@ fn sign(i: &[u8]) -> IResult<&[u8], i32> {
     })(i)
 }
 
-fn fractions(i: &[u8]) -> IResult<&[u8], f32> {
-    let (i, digits) = take_while(is_digit)(i)?;
-    let digits = str::from_utf8(digits).unwrap(); // This can't panic, `digits` will only include digits.
-    let f = format!("0.{}", digits).parse().unwrap(); // This can't panic, the string is a valid `f32`.
-
-    Ok((i, f))
-}
-
 // DATE
 
 // [+/-]YYYY
@@ -183,6 +149,9 @@ fn date_iso_week(i: &[u8]) -> IResult<&[u8], Date> {
     )(i)
 }
 
+/// Parses a date string.
+///
+/// See [`date`][`crate::date`] for the supported formats.
 pub fn parse_date(i: &[u8]) -> IResult<&[u8], Date> {
     alt((date_ymd, date_iso_week, date_ordinal))(i)
 }
@@ -204,20 +173,40 @@ fn time_second(i: &[u8]) -> IResult<&[u8], u32> {
     n_digit_in_range(i, 2, 0..=60)
 }
 
-fn time_millisecond(fraction: f32) -> u32 {
-    (1000.0 * fraction) as u32
+// Converts the fractional part if-any of a number of seconds to milliseconds
+// truncating towards zero if there are more than three digits.
+// e.g. "" -> 0, "1" -> 100, "12" -> 120, "123" -> 123, "1234" -> 123
+fn fraction_millisecond(i: &[u8]) -> IResult<&[u8], u32> {
+    let (i, mut digits) = take_while(is_digit)(i)?;
+    let mut l = digits.len();
+    if l > 3 {
+        digits = digits.get(0..3).unwrap();
+    }
+    let mut result = 0;
+    if l > 0 {
+        let digits = str::from_utf8(digits).unwrap(); // This can't panic, `digits` will only include digits.
+        result = digits.parse().unwrap();
+    }
+    while l < 3 {
+        result *= 10;
+        l += 1;
+    }
+    Ok((i, result))
 }
 
+/// Parses a time string.
+///
+/// See [`time`][`crate::time`] for the supported formats.
 // HH:MM:[SS][.(m*)][(Z|+...|-...)]
 pub fn parse_time(i: &[u8]) -> IResult<&[u8], Time> {
     map(
         tuple((
-            time_hour,                                                     // HH
-            opt(tag(b":")),                                                // :
-            time_minute,                                                   // MM
-            opt(preceded(opt(tag(b":")), time_second)),                    // [SS]
-            opt(map(preceded(one_of(",."), fractions), time_millisecond)), // [.(m*)]
-            opt(alt((timezone_hour, timezone_utc))),                       // [(Z|+...|-...)]
+            time_hour,                                         // HH
+            opt(tag(b":")),                                    // :
+            time_minute,                                       // MM
+            opt(preceded(opt(tag(b":")), time_second)),        // [SS]
+            opt(preceded(one_of(",."), fraction_millisecond)), // [.(m*)]
+            opt(alt((timezone_hour, timezone_utc))),           // [(Z|+...|-...)]
         )),
         |(h, _, m, s, ms, z)| {
             let (tz_offset_hours, tz_offset_minutes) = z.unwrap_or((0, 0));
@@ -245,6 +234,9 @@ fn timezone_utc(i: &[u8]) -> IResult<&[u8], (i32, i32)> {
     map(tag(b"Z"), |_| (0, 0))(i)
 }
 
+/// Parses a datetime string.
+///
+/// See [`datetime`][`crate::datetime`] for supported formats.
 // Full ISO8601 datetime
 pub fn parse_datetime(i: &[u8]) -> IResult<&[u8], DateTime> {
     map(
@@ -255,54 +247,61 @@ pub fn parse_datetime(i: &[u8]) -> IResult<&[u8], DateTime> {
 
 // DURATION
 
-// Y[YYY...]
+///    dur-year          = 1*DIGIT "Y" [dur-month]
 fn duration_year(i: &[u8]) -> IResult<&[u8], u32> {
-    take_digits(i)
+    terminated(take_digits, tag(b"Y"))(i)
 }
 
-// M[M]
+///    dur-month         = 1*DIGIT "M" [dur-day]
 fn duration_month(i: &[u8]) -> IResult<&[u8], u32> {
-    m_to_n_digit_in_range(i, 1, 2, 0..=12)
+    terminated(take_digits, tag(b"M"))(i)
 }
 
-// W[W]
+///    dur-week          = 1*DIGIT "W"
 fn duration_week(i: &[u8]) -> IResult<&[u8], u32> {
-    m_to_n_digit_in_range(i, 1, 2, 0..=52)
+    terminated(take_digits, tag(b"W"))(i)
 }
 
-// D[D]
+//    dur-day           = 1*DIGIT "D"
 fn duration_day(i: &[u8]) -> IResult<&[u8], u32> {
-    m_to_n_digit_in_range(i, 1, 2, 0..=31)
+    terminated(take_digits, tag(b"D"))(i)
 }
 
-// H[H]
+///    dur-hour          = 1*DIGIT "H" [dur-minute]
+///    dur-time          = "T" (dur-hour / dur-minute / dur-second)
 fn duration_hour(i: &[u8]) -> IResult<&[u8], u32> {
-    m_to_n_digit_in_range(i, 1, 2, 0..=24)
+    terminated(take_digits, tag(b"H"))(i)
 }
 
-// M[M]
+///    dur-minute        = 1*DIGIT "M" [dur-second]
 fn duration_minute(i: &[u8]) -> IResult<&[u8], u32> {
-    m_to_n_digit_in_range(i, 1, 2, 0..=60)
+    terminated(take_digits, tag(b"M"))(i)
 }
 
-// S[S][[,.][MS]]
+////    dur-second        = 1*DIGIT "S"
+fn duration_second(i: &[u8]) -> IResult<&[u8], u32> {
+    terminated(take_digits, tag(b"S"))(i)
+}
+
+///    dur-second-ext    = 1*DIGIT (,|.) 1*DIGIT "S"
 fn duration_second_and_millisecond(i: &[u8]) -> IResult<&[u8], (u32, u32)> {
-    let (i, s) = m_to_n_digit_in_range(i, 1, 2, 0..=60)?;
-    let (i, ms) = opt(map(preceded(one_of(",."), fractions), duration_millisecond))(i)?;
-
-    Ok((i, (s, ms.unwrap_or(0))))
-}
-
-fn duration_millisecond(fraction: f32) -> u32 {
-    (1000.0 * fraction) as u32
+    alt((
+        // no milliseconds
+        map(duration_second, |m| (m, 0)),
+        terminated(
+            // with milliseconds
+            separated_pair(take_digits, one_of(",."), fraction_millisecond),
+            tag(b"S"),
+        ),
+    ))(i)
 }
 
 fn duration_time(i: &[u8]) -> IResult<&[u8], (u32, u32, u32, u32)> {
     map(
         tuple((
-            opt(terminated(duration_hour, tag(b"H"))),
-            opt(terminated(duration_minute, tag(b"M"))),
-            opt(terminated(duration_second_and_millisecond, tag(b"S"))),
+            opt(duration_hour),
+            opt(duration_minute),
+            opt(duration_second_and_millisecond),
         )),
         |(h, m, s)| {
             let (s, ms) = s.unwrap_or((0, 0));
@@ -317,9 +316,9 @@ fn duration_ymdhms(i: &[u8]) -> IResult<&[u8], Duration> {
         preceded(
             tag(b"P"),
             tuple((
-                opt(terminated(duration_year, tag(b"Y"))),
-                opt(terminated(duration_month, tag(b"M"))),
-                opt(terminated(duration_day, tag(b"D"))),
+                opt(duration_year),
+                opt(duration_month),
+                opt(duration_day),
                 opt(preceded(tag(b"T"), duration_time)),
             )),
         ),
@@ -345,10 +344,7 @@ fn duration_ymdhms(i: &[u8]) -> IResult<&[u8], Duration> {
 }
 
 fn duration_weeks(i: &[u8]) -> IResult<&[u8], Duration> {
-    map(
-        preceded(tag(b"P"), terminated(duration_week, tag(b"W"))),
-        Duration::Weeks,
-    )(i)
+    map(preceded(tag(b"P"), duration_week), Duration::Weeks)(i)
 }
 
 // YYYY, no sign
@@ -382,6 +378,9 @@ fn duration_datetime(i: &[u8]) -> IResult<&[u8], Duration> {
     )(i)
 }
 
+/// Parses a duration string.
+///
+/// See [`duration`][`crate::duration`] for supported formats.
 pub fn parse_duration(i: &[u8]) -> IResult<&[u8], Duration> {
     alt((duration_ymdhms, duration_weeks, duration_datetime))(i)
 }
